@@ -1,19 +1,19 @@
 /*
  * Copyright © 2001 Havoc Pennington
- * Copyright © 2008 Christian Persch
+ * Copyright © 2008, 2010 Christian Persch
  *
- * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Library General Public
+ * This programme is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public
  * License as published by the Free Software Foundation; either
  * version 3 of the License, or (at your option) any later version.
  *
- * This library is distributed in the hope that it will be useful,
+ * This programme is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Library General Public License for more details.
+ * General Public License for more details.
  *
- * You should have received a copy of the GNU Library General Public
- * License along with this library; if not, write to the
+ * You should have received a copy of the GNU General Public
+ * License along with this programme; if not, write to the
  * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA.
  */
@@ -21,16 +21,13 @@
 #ifndef TERMINAL_UTIL_H
 #define TERMINAL_UTIL_H
 
+#include <gio/gio.h>
 #include <gtk/gtk.h>
-#include <gconf/gconf-client.h>
 
 #include "terminal-screen.h"
 
 G_BEGIN_DECLS
 
-#define CONF_PROXY_PREFIX      "/system/proxy"
-#define CONF_HTTP_PROXY_PREFIX "/system/http_proxy"
-
 void terminal_util_set_unique_role (GtkWindow *window, const char *prefix);
 
 void terminal_util_show_error_dialog (GtkWindow *transient_parent, 
@@ -40,6 +37,8 @@ void terminal_util_show_error_dialog (GtkWindow *transient_parent,
 
 void terminal_util_show_help (const char *topic, GtkWindow  *transient_parent);
 
+void terminal_util_show_about (GtkWindow *transient_parent);
+
 void terminal_util_set_labelled_by          (GtkWidget  *widget,
                                              GtkLabel   *label);
 void terminal_util_set_atk_name_description (GtkWidget  *widget,
@@ -51,9 +50,6 @@ void terminal_util_open_url (GtkWidget *parent,
                              TerminalURLFlavour flavor,
                              guint32 user_time);
 
-char *terminal_util_resolve_relative_path (const char *path,
-                                           const char *relative_path);
-
 void terminal_util_transform_uris_to_quoted_fuse_paths (char **uris);
 
 char *terminal_util_concat_uris (char **uris,
@@ -61,9 +57,9 @@ char *terminal_util_concat_uris (char **uris,
 
 char *terminal_util_get_licence_text (void);
 
-gboolean terminal_util_load_builder_file (const char *filename,
-                                          const char *object_name,
-                                          ...);
+gboolean terminal_util_load_builder_resource (const char *path,
+                                              const char *object_name,
+                                              ...);
 
 gboolean terminal_util_dialog_response_on_delete (GtkWindow *widget);
 
@@ -71,32 +67,16 @@ void terminal_util_key_file_set_string_escape    (GKeyFile *key_file,
                                                   const char *group,
                                                   const char *key,
                                                   const char *string);
-char *terminal_util_key_file_get_string_unescape (GKeyFile *key_file,
-                                                  const char *group,
-                                                  const char *key,
-                                                  GError **error);
-
 void terminal_util_key_file_set_argv      (GKeyFile *key_file,
                                            const char *group,
                                            const char *key,
                                            int argc,
                                            char **argv);
-char **terminal_util_key_file_get_argv    (GKeyFile *key_file,
-                                           const char *group,
-                                           const char *key,
-                                           int *argc,
-                                           GError **error);
 
 void terminal_util_add_proxy_env (GHashTable *env_table);
 
-typedef enum {
-  FLAG_INVERT_BOOL  = 1 << 0,
-} PropertyChangeFlags;
-
-void terminal_util_bind_object_property_to_widget (GObject *object,
-                                                   const char *object_prop,
-                                                   GtkWidget *widget,
-                                                   PropertyChangeFlags flags);
+GdkScreen *terminal_util_get_screen_by_display_name (const char *display_name,
+                                                     int screen_number);
 
 gboolean terminal_util_x11_get_net_wm_desktop (GdkWindow *window,
 					       guint32   *desktop);
@@ -107,6 +87,23 @@ void terminal_util_x11_clear_demands_attention (GdkWindow *window);
 
 gboolean terminal_util_x11_window_is_minimized (GdkWindow *window);
 
+const GdkRGBA *terminal_g_settings_get_rgba (GSettings  *settings,
+                                             const char *key,
+                                             GdkRGBA    *rgba);
+void terminal_g_settings_set_rgba (GSettings  *settings,
+                                   const char *key,
+                                   const GdkRGBA *rgba);
+
+GdkRGBA *terminal_g_settings_get_rgba_palette (GSettings  *settings,
+                                               const char *key,
+                                               gsize      *n_colors);
+void terminal_g_settings_set_rgba_palette (GSettings      *settings,
+                                           const char     *key,
+                                           const GdkRGBA  *colors,
+                                           gsize           n_colors);
+
+void terminal_util_bind_mnemonic_label_sensitivity (GtkWidget *widget);
+
 G_END_DECLS
 
 #endif /* TERMINAL_UTIL_H */
